@@ -28,13 +28,15 @@
 # Copyright (c) 2020 Anthony Sottile <asottile@umich.edu>
 # Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 # Copyright (c) 2021 Matus Valo <matusvalo@users.noreply.github.com>
+# Copyright (c) 2021 victor <16359131+jiajunsu@users.noreply.github.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/master/LICENSE
+# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
 
 """Checkers for various standard library functions."""
 
 import sys
+from collections.abc import Iterable
 
 import astroid
 
@@ -51,7 +53,23 @@ SUBPROCESS_POPEN = "subprocess.Popen"
 SUBPROCESS_RUN = "subprocess.run"
 OPEN_MODULE = "_io"
 
+
+DEPRECATED_MODULES = {
+    (0, 0, 0): {"tkinter.tix", "fpectl"},
+    (3, 2, 0): {"optparse"},
+    (3, 4, 0): {"imp"},
+    (3, 5, 0): {"formatter"},
+    (3, 6, 0): {"asynchat", "asyncore"},
+    (3, 7, 0): {"macpath"},
+    (3, 9, 0): {"lib2to3", "parser", "symbol", "binhex"},
+}
+
 DEPRECATED_ARGUMENTS = {
+    (0, 0, 0): {
+        "int": ((None, "x"),),
+        "bool": ((None, "x"),),
+        "float": ((None, "x"),),
+    },
     (3, 8, 0): {
         "asyncio.tasks.sleep": ((None, "loop"),),
         "asyncio.tasks.gather": ((None, "loop"),),
@@ -63,6 +81,8 @@ DEPRECATED_ARGUMENTS = {
         "asyncio.subprocess.create_subprocess_shell": ((4, "loop"),),
         "gettext.translation": ((5, "codeset"),),
         "gettext.install": ((2, "codeset"),),
+        "functools.partialmethod": ((None, "func"),),
+        "weakref.finalize": ((None, "func"), (None, "obj")),
         "profile.Profile.runcall": ((None, "func"),),
         "cProfile.Profile.runcall": ((None, "func"),),
         "bdb.Bdb.runcall": ((None, "func"),),
@@ -82,6 +102,15 @@ DEPRECATED_ARGUMENTS = {
     (3, 9, 0): {"random.Random.shuffle": ((1, "random"),)},
 }
 
+DEPRECATED_DECORATORS = {
+    (3, 8, 0): {"asyncio.coroutine"},
+    (3, 3, 0): {
+        "abc.abstractclassmethod",
+        "abc.abstractstaticmethod",
+        "abc.abstractproperty",
+    },
+}
+
 
 DEPRECATED_METHODS = {
     0: {
@@ -137,6 +166,7 @@ DEPRECATED_METHODS = {
             "base64.decodestring",
             "ntpath.splitunc",
             "os.path.splitunc",
+            "os.stat_float_times",
         },
         (3, 2, 0): {
             "cgi.escape",
@@ -153,6 +183,7 @@ DEPRECATED_METHODS = {
             "logging.LoggerAdapter.warn",
             "nntplib._NNTPBase.xpath",
             "platform.popen",
+            "sqlite3.OptimizedUnicode",
         },
         (3, 4, 0): {
             "importlib.find_loader",
@@ -169,15 +200,20 @@ DEPRECATED_METHODS = {
             "platform.linux_distribution",
             "platform.dist",
         },
-        (3, 6, 0): {"importlib._bootstrap_external.FileLoader.load_module"},
+        (3, 6, 0): {
+            "importlib._bootstrap_external.FileLoader.load_module",
+            "_ssl.RAND_pseudo_bytes",
+        },
         (3, 7, 0): {
             "sys.set_coroutine_wrapper",
             "sys.get_coroutine_wrapper",
             "aifc.openfp",
+            "threading.Thread.isAlive",
             "asyncio.Task.current_task",
             "asyncio.Task.all_task",
             "locale.format",
             "ssl.wrap_socket",
+            "ssl.match_hostname",
             "sunau.openfp",
             "wave.openfp",
         },
@@ -197,6 +233,67 @@ DEPRECATED_METHODS = {
             "binascii.rlecode_hqx",
             "binascii.rledecode_hqx",
         },
+        (3, 10, 0): {
+            "_sqlite3.enable_shared_cache",
+            "pathlib.Path.link_to",
+            "zipimport.zipimporter.load_module",
+            "zipimport.zipimporter.find_module",
+            "zipimport.zipimporter.find_loader",
+            "threading.currentThread",
+            "threading.activeCount",
+            "threading.Condition.notifyAll",
+            "threading.Event.isSet",
+            "threading.Thread.setName",
+            "threading.Thread.getName",
+            "threading.Thread.isDaemon",
+            "threading.Thread.setDaemon",
+            "cgi.log",
+        },
+    },
+}
+
+
+DEPRECATED_CLASSES = {
+    (3, 3, 0): {
+        "importlib.abc": {
+            "Finder",
+        },
+        "pkgutil": {
+            "ImpImporter",
+            "ImpLoader",
+        },
+        "collections": {
+            "Awaitable",
+            "Coroutine",
+            "AsyncIterable",
+            "AsyncIterator",
+            "AsyncGenerator",
+            "Hashable",
+            "Iterable",
+            "Iterator",
+            "Generator",
+            "Reversible",
+            "Sized",
+            "Container",
+            "Callable",
+            "Collection",
+            "Set",
+            "MutableSet",
+            "Mapping",
+            "MutableMapping",
+            "MappingView",
+            "KeysView",
+            "ItemsView",
+            "ValuesView",
+            "Sequence",
+            "MutableSequence",
+            "ByteString",
+        },
+    },
+    (3, 9, 0): {
+        "smtpd": {
+            "MailmanProxy",
+        }
     },
 }
 
@@ -316,6 +413,16 @@ class StdlibChecker(DeprecatedMixin, BaseChecker):
             "deprecated-argument",
             "The argument is marked as deprecated and will be removed in the future.",
         ),
+        "W1512": (
+            "Using deprecated class %s of module %s",
+            "deprecated-class",
+            "The class is marked as deprecated and will be removed in the future.",
+        ),
+        "W1513": (
+            "Using deprecated decorator %s()",
+            "deprecated-decorator",
+            "The decorator is marked as deprecated and will be removed in the future.",
+        ),
     }
 
     def __init__(self, linter=None):
@@ -329,6 +436,18 @@ class StdlibChecker(DeprecatedMixin, BaseChecker):
         for since_vers, func_list in DEPRECATED_ARGUMENTS.items():
             if since_vers <= sys.version_info:
                 self._deprecated_attributes.update(func_list)
+        self._deprecated_classes = dict()
+        for since_vers, class_list in DEPRECATED_CLASSES.items():
+            if since_vers <= sys.version_info:
+                self._deprecated_classes.update(class_list)
+        self._deprecated_modules = set()
+        for since_vers, mod_list in DEPRECATED_MODULES.items():
+            if since_vers <= sys.version_info:
+                self._deprecated_modules.update(mod_list)
+        self._deprecated_decorators = set()
+        for since_vers, decorator_list in DEPRECATED_DECORATORS.items():
+            if since_vers <= sys.version_info:
+                self._deprecated_decorators.update(decorator_list)
 
     def _check_bad_thread_instantiation(self, node):
         if not node.kwargs and not node.keywords and len(node.args) <= 1:
@@ -363,10 +482,12 @@ class StdlibChecker(DeprecatedMixin, BaseChecker):
         "invalid-envvar-default",
         "subprocess-popen-preexec-fn",
         "subprocess-run-check",
+        "deprecated-class",
     )
     def visit_call(self, node):
         """Visit a Call node."""
         try:
+            self.check_deprecated_class_in_call(node)
             for inferred in node.func.infer():
                 if inferred is astroid.Uninferable:
                     continue
@@ -505,12 +626,22 @@ class StdlibChecker(DeprecatedMixin, BaseChecker):
         else:
             self.add_message(message, node=node, args=(name, call_arg.pytype()))
 
+    def deprecated_modules(self):
+        """Callback returning the deprecated modules."""
+        return self._deprecated_modules
+
     def deprecated_methods(self):
         return self._deprecated_methods
 
     def deprecated_arguments(self, method: str):
         return self._deprecated_attributes.get(method, ())
 
+    def deprecated_classes(self, module: str):
+        return self._deprecated_classes.get(module, ())
+
+    def deprecated_decorators(self) -> Iterable:
+        return self._deprecated_decorators
+
 
 def register(linter):
     """required method to auto register this checker"""
