@@ -43,11 +43,13 @@
 # Copyright (c) 2020 Anthony Sottile <asottile@umich.edu>
 # Copyright (c) 2020 Anubhav <35621759+anubh-v@users.noreply.github.com>
 # Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
+# Copyright (c) 2021 yushao2 <36848472+yushao2@users.noreply.github.com>
+# Copyright (c) 2021 Andrew Haigh <nelfin@gmail.com>
 # Copyright (c) 2021 Jens H. Nielsen <Jens.Nielsen@microsoft.com>
 # Copyright (c) 2021 Ikraduya Edian <ikraduya@gmail.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/master/LICENSE
+# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
 
 """try to find more bugs in the code using astroid inference capabilities
 """
@@ -397,6 +399,11 @@ MSGS = {
         "dict-iter-missing-items",
         "Emitted when trying to iterate through a dict without calling .items()",
     ),
+    "E1142": (
+        "'await' should be used within an async function",
+        "await-outside-async",
+        "Emitted when await is used outside an async function.",
+    ),
     "W1113": (
         "Keyword argument before variable positional arguments list "
         "in the definition of %s function",
@@ -458,7 +465,9 @@ def _emit_no_member(node, owner, owner_name, ignored_mixins=True, ignored_none=T
         return False
     if owner_name and ignored_mixins and owner_name[-5:].lower() == "mixin":
         return False
-    if isinstance(owner, astroid.FunctionDef) and owner.decorators:
+    if isinstance(owner, astroid.FunctionDef) and (
+        owner.decorators or owner.is_abstract()
+    ):
         return False
     if isinstance(owner, (astroid.Instance, astroid.ClassDef)):
         if owner.has_dynamic_getattr():
@@ -469,7 +478,8 @@ def _emit_no_member(node, owner, owner_name, ignored_mixins=True, ignored_none=T
             except astroid.MroError:
                 return False
             if metaclass:
-                return metaclass.qname() == "enum.EnumMeta"
+                # Renamed in Python 3.10 to `EnumType`
+                return metaclass.qname() in ("enum.EnumMeta", "enum.EnumType")
             return False
         if not has_known_bases(owner):
             return False
@@ -1436,8 +1446,8 @@ accessed. Python regular expressions are accepted.",
                         args=(display_name, callable_name),
                     )
 
-        for name in kwparams:
-            defval, assigned = kwparams[name]
+        for name, val in kwparams.items():
+            defval, assigned = val
             if (
                 defval is None
                 and not assigned
@@ -1767,11 +1777,7 @@ accessed. Python regular expressions are accepted.",
 
         if isinstance(node.value, astroid.Dict):
             # Assert dict key is hashable
-            if isinstance(node.slice, (astroid.Index, astroid.Slice)):
-                # In Python 3.9 the Index, Slice and ExtSlice nodes are no longer in use
-                inferred = safe_infer(node.slice.value)
-            else:
-                inferred = safe_infer(node.slice)
+            inferred = safe_infer(node.slice)
             if inferred not in (None, astroid.Uninferable):
                 try:
                     hash_fn = next(inferred.igetattr("__hash__"))
@@ -1948,6 +1954,20 @@ class IterableChecker(BaseChecker):
         for gen in node.generators:
             self._check_iterable(gen.iter, check_async=gen.is_async)
 
+    @check_messages("await-outside-async")
+    def visit_await(self, node: astroid.Await) -> None:
+        self._check_await_outside_coroutine(node)
+
+    def _check_await_outside_coroutine(self, node: astroid.Await) -> None:
+        node_scope = node.scope()
+        while not isinstance(node_scope, astroid.Module):
+            if isinstance(node_scope, astroid.AsyncFunctionDef):
+                return
+            if isinstance(node_scope, astroid.FunctionDef):
+                break
+            node_scope = node_scope.parent.scope()
+        self.add_message("await-outside-async", node=node)
+
 
 def register(linter):
     """required method to auto register this checker"""
