@@ -1,24 +1,28 @@
-# -*- coding: utf-8 -*-
-# Copyright (c) 2016-2018 Ashley Whetter <ashley@awhetter.co.uk>
-# Copyright (c) 2016-2017 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2016-2020 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2016-2019 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2016 Yuri Bochkarev <baltazar.bz@gmail.com>
 # Copyright (c) 2016 Glenn Matthews <glenn@e-dad.net>
 # Copyright (c) 2016 Moises Lopez <moylop260@vauxoo.com>
-# Copyright (c) 2017 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2017, 2020 hippo91 <guillaume.peillex@gmail.com>
 # Copyright (c) 2017 Mitar <mitar.github@tnode.com>
+# Copyright (c) 2018, 2020 Anthony Sottile <asottile@umich.edu>
+# Copyright (c) 2018 Jim Robertson <jrobertson98atx@gmail.com>
 # Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
-# Copyright (c) 2018 Anthony Sottile <asottile@umich.edu>
 # Copyright (c) 2018 Mitchell T.H. Young <mitchelly@gmail.com>
 # Copyright (c) 2018 Adrian Chirieac <chirieacam@gmail.com>
+# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
+# Copyright (c) 2019 Danny Hermes <daniel.j.hermes@gmail.com>
+# Copyright (c) 2019 Zeb Nicholls <zebedee.nicholls@climate-energy-college.org>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
+# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/master/COPYING
+# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
 
 """Utility methods for docstring checking."""
 
-from __future__ import absolute_import, print_function
-
 import re
+from typing import List
 
 import astroid
 
@@ -33,7 +37,7 @@ def space_indentation(s):
     :rtype: int
     :return: number of leading spaces
     """
-    return len(s) - len(s.lstrip(' '))
+    return len(s) - len(s.lstrip(" "))
 
 
 def get_setters_property_name(node):
@@ -48,9 +52,11 @@ def get_setters_property_name(node):
     """
     decorators = node.decorators.nodes if node.decorators else []
     for decorator in decorators:
-        if (isinstance(decorator, astroid.Attribute) and
-                decorator.attrname == "setter" and
-                isinstance(decorator.expr, astroid.Name)):
+        if (
+            isinstance(decorator, astroid.Attribute)
+            and decorator.attrname == "setter"
+            and isinstance(decorator.expr, astroid.Name)
+        ):
             return decorator.expr.name
     return None
 
@@ -97,6 +103,19 @@ def returns_something(return_node):
     return not (isinstance(returns, astroid.Const) and returns.value is None)
 
 
+def _get_raise_target(node):
+    if isinstance(node.exc, astroid.Call):
+        func = node.exc.func
+        if isinstance(func, (astroid.Name, astroid.Attribute)):
+            return utils.safe_infer(func)
+    return None
+
+
+def _split_multiple_exc_types(target: str) -> List[str]:
+    delimiters = r"(\s*,(?:\s*or\s)?\s*|\s+or\s+)"
+    return re.split(delimiters, target)
+
+
 def possible_exc_types(node):
     """
     Gets all of the possible raised exception types for the given raise node.
@@ -117,9 +136,16 @@ def possible_exc_types(node):
         inferred = utils.safe_infer(node.exc)
         if inferred:
             excs = [inferred.name]
-    elif (isinstance(node.exc, astroid.Call) and
-          isinstance(node.exc.func, astroid.Name)):
-        target = utils.safe_infer(node.exc.func)
+    elif node.exc is None:
+        handler = node.parent
+        while handler and not isinstance(handler, astroid.ExceptHandler):
+            handler = handler.parent
+
+        if handler and handler.type:
+            inferred_excs = astroid.unpack_infer(handler.type)
+            excs = (exc.name for exc in inferred_excs if exc is not astroid.Uninferable)
+    else:
+        target = _get_raise_target(node)
         if isinstance(target, astroid.ClassDef):
             excs = [target.name]
         elif isinstance(target, astroid.FunctionDef):
@@ -129,19 +155,12 @@ def possible_exc_types(node):
                     continue
 
                 val = utils.safe_infer(ret.value)
-                if (val and isinstance(val, (astroid.Instance, astroid.ClassDef))
-                        and utils.inherit_from_std_ex(val)):
+                if (
+                    val
+                    and isinstance(val, (astroid.Instance, astroid.ClassDef))
+                    and utils.inherit_from_std_ex(val)
+                ):
                     excs.append(val.name)
-    elif node.exc is None:
-        handler = node.parent
-        while handler and not isinstance(handler, astroid.ExceptHandler):
-            handler = handler.parent
-
-        if handler and handler.type:
-            inferred_excs = astroid.unpack_infer(handler.type)
-            excs = (exc.name for exc in inferred_excs
-                    if exc is not astroid.Uninferable)
-
 
     try:
         return {exc for exc in excs if not utils.node_ignores_exception(node, exc)}
@@ -149,9 +168,13 @@ def possible_exc_types(node):
         return set()
 
 
-def docstringify(docstring, default_type='default'):
-    for docstring_type in [SphinxDocstring, EpytextDocstring,
-                           GoogleDocstring, NumpyDocstring]:
+def docstringify(docstring, default_type="default"):
+    for docstring_type in (
+        SphinxDocstring,
+        EpytextDocstring,
+        GoogleDocstring,
+        NumpyDocstring,
+    ):
         instance = docstring_type(docstring)
         if instance.is_valid():
             return instance
@@ -161,11 +184,14 @@ def docstringify(docstring, default_type='default'):
 
 
 class Docstring:
-    re_for_parameters_see = re.compile(r"""
+    re_for_parameters_see = re.compile(
+        r"""
         For\s+the\s+(other)?\s*parameters\s*,\s+see
-        """, re.X | re.S)
+        """,
+        re.X | re.S,
+    )
 
-    supports_yields = None
+    supports_yields: bool = False
     """True if the docstring supports a "yield" section.
 
     False if the docstring uses the returns section to document generators.
@@ -177,6 +203,9 @@ class Docstring:
         doc = doc or ""
         self.doc = doc.expandtabs()
 
+    def __repr__(self) -> str:
+        return f"<{self.__class__.__name__}:'''{self.doc}'''>"
+
     def is_valid(self):
         return False
 
@@ -212,17 +241,31 @@ class Docstring:
 
 
 class SphinxDocstring(Docstring):
-    re_type = r"[\w\.]+"
+    re_type = r"""
+        [~!.]?               # Optional link style prefix
+        \w(?:\w|\.[^\.])*    # Valid python name
+        """
 
     re_simple_container_type = r"""
         {type}                        # a container type
         [\(\[] [^\n\s]+ [\)\]]        # with the contents of the container
-    """.format(type=re_type)
+    """.format(
+        type=re_type
+    )
+
+    re_multiple_simple_type = r"""
+        (?:{container_type}|{type})
+        (?:(?:\s+(?:of|or)\s+|\s*,\s*)(?:{container_type}|{type}))*
+    """.format(
+        type=re_type, container_type=re_simple_container_type
+    )
 
     re_xref = r"""
         (?::\w+:)?                    # optional tag
         `{}`                         # what to reference
-        """.format(re_type)
+        """.format(
+        re_type
+    )
 
     re_param_raw = r"""
         :                       # initial colon
@@ -241,7 +284,9 @@ class SphinxDocstring(Docstring):
         (\w+)                   # Parameter name
         \s*                     # whitespace
         :                       # final colon
-        """.format(type=re_type, container_type=re_simple_container_type)
+        """.format(
+        type=re_type, container_type=re_simple_container_type
+    )
     re_param_in_docstring = re.compile(re_param_raw, re.X | re.S)
 
     re_type_raw = r"""
@@ -250,17 +295,19 @@ class SphinxDocstring(Docstring):
         ({type})                # Parameter name
         \s*                     # whitespace
         :                       # final colon
-        """.format(type=re_type)
+        """.format(
+        type=re_multiple_simple_type
+    )
     re_type_in_docstring = re.compile(re_type_raw, re.X | re.S)
 
     re_property_type_raw = r"""
         :type:                  # Sphinx keyword
         \s+                     # whitespace
         {type}                  # type declaration
-        """.format(type=re_type)
-    re_property_type_in_docstring = re.compile(
-        re_property_type_raw, re.X | re.S
+        """.format(
+        type=re_multiple_simple_type
     )
+    re_property_type_in_docstring = re.compile(re_property_type_raw, re.X | re.S)
 
     re_raise_raw = r"""
         :                       # initial colon
@@ -269,16 +316,12 @@ class SphinxDocstring(Docstring):
         except|exception
         )
         \s+                     # whitespace
-
-        (?:                     # type declaration
-        ({type})
-        \s+
-        )?
-
-        (\w+)                   # Parameter name
+        ({type})                # exception type
         \s*                     # whitespace
         :                       # final colon
-        """.format(type=re_type)
+        """.format(
+        type=re_multiple_simple_type
+    )
     re_raise_in_docstring = re.compile(re_raise_raw, re.X | re.S)
 
     re_rtype_in_docstring = re.compile(r":rtype:")
@@ -288,18 +331,20 @@ class SphinxDocstring(Docstring):
     supports_yields = False
 
     def is_valid(self):
-        return bool(self.re_param_in_docstring.search(self.doc) or
-                    self.re_raise_in_docstring.search(self.doc) or
-                    self.re_rtype_in_docstring.search(self.doc) or
-                    self.re_returns_in_docstring.search(self.doc) or
-                    self.re_property_type_in_docstring.search(self.doc))
+        return bool(
+            self.re_param_in_docstring.search(self.doc)
+            or self.re_raise_in_docstring.search(self.doc)
+            or self.re_rtype_in_docstring.search(self.doc)
+            or self.re_returns_in_docstring.search(self.doc)
+            or self.re_property_type_in_docstring.search(self.doc)
+        )
 
     def exceptions(self):
         types = set()
 
         for match in re.finditer(self.re_raise_in_docstring, self.doc):
-            raise_type = match.group(2)
-            types.add(raise_type)
+            raise_type = match.group(1)
+            types.update(_split_multiple_exc_types(raise_type))
 
         return types
 
@@ -327,7 +372,7 @@ class SphinxDocstring(Docstring):
 
         # The summary line is the return doc,
         # so the first line must not be a known directive.
-        return not self.doc.lstrip().startswith(':')
+        return not self.doc.lstrip().startswith(":")
 
     def has_property_type(self):
         if not self.doc:
@@ -360,29 +405,33 @@ class EpytextDocstring(SphinxDocstring):
         https://www.jetbrains.com/help/pycharm/2016.1/creating-documentation-comments.html#d848203e314
         https://www.jetbrains.com/help/pycharm/2016.1/using-docstrings-to-specify-types.html
     """
+
     re_param_in_docstring = re.compile(
-        SphinxDocstring.re_param_raw.replace(':', '@', 1),
-        re.X | re.S)
+        SphinxDocstring.re_param_raw.replace(":", "@", 1), re.X | re.S
+    )
 
     re_type_in_docstring = re.compile(
-        SphinxDocstring.re_type_raw.replace(':', '@', 1),
-        re.X | re.S)
+        SphinxDocstring.re_type_raw.replace(":", "@", 1), re.X | re.S
+    )
 
     re_property_type_in_docstring = re.compile(
-        SphinxDocstring.re_property_type_raw.replace(':', '@', 1),
-        re.X | re.S)
+        SphinxDocstring.re_property_type_raw.replace(":", "@", 1), re.X | re.S
+    )
 
     re_raise_in_docstring = re.compile(
-        SphinxDocstring.re_raise_raw.replace(':', '@', 1),
-        re.X | re.S)
+        SphinxDocstring.re_raise_raw.replace(":", "@", 1), re.X | re.S
+    )
 
-    re_rtype_in_docstring = re.compile(r"""
+    re_rtype_in_docstring = re.compile(
+        r"""
         @                       # initial "at" symbol
         (?:                     # Epytext keyword
         rtype|returntype
         )
         :                       # final colon
-        """, re.X | re.S)
+        """,
+        re.X | re.S,
+    )
 
     re_returns_in_docstring = re.compile(r"@returns?:")
 
@@ -394,7 +443,7 @@ class EpytextDocstring(SphinxDocstring):
         if self.has_property_type():
             # The summary line is the return doc,
             # so the first line must not be a known directive.
-            return not self.doc.lstrip().startswith('@')
+            return not self.doc.lstrip().startswith("@")
 
         return False
 
@@ -407,12 +456,16 @@ class GoogleDocstring(Docstring):
     re_container_type = r"""
         (?:{type}|{xref})             # a container type
         [\(\[] [^\n]+ [\)\]]          # with the contents of the container
-    """.format(type=re_type, xref=re_xref)
+    """.format(
+        type=re_type, xref=re_xref
+    )
 
     re_multiple_type = r"""
         (?:{container_type}|{type}|{xref})
-        (?:\s+or\s+(?:{container_type}|{type}|{xref}))*
-    """.format(type=re_type, xref=re_xref, container_type=re_container_type)
+        (?:(?:\s+(?:of|or)\s+|\s*,\s*)(?:{container_type}|{type}|{xref}))*
+    """.format(
+        type=re_type, xref=re_xref, container_type=re_container_type
+    )
 
     _re_section_template = r"""
         ^([ ]*)   {0} \s*:   \s*$     # Google parameter header
@@ -421,15 +474,16 @@ class GoogleDocstring(Docstring):
 
     re_param_section = re.compile(
         _re_section_template.format(r"(?:Args|Arguments|Parameters)"),
-        re.X | re.S | re.M
+        re.X | re.S | re.M,
     )
 
     re_keyword_param_section = re.compile(
         _re_section_template.format(r"Keyword\s(?:Args|Arguments|Parameters)"),
-        re.X | re.S | re.M
+        re.X | re.S | re.M,
     )
 
-    re_param_line = re.compile(r"""
+    re_param_line = re.compile(
+        r"""
         \s*  \*{{0,2}}(\w+)             # identifier potentially with asterisks
         \s*  ( [(]
             {type}
@@ -437,41 +491,51 @@ class GoogleDocstring(Docstring):
             [)] )? \s* :                # optional type declaration
         \s*  (.*)                       # beginning of optional description
     """.format(
-        type=re_multiple_type,
-    ), re.X | re.S | re.M)
+            type=re_multiple_type
+        ),
+        re.X | re.S | re.M,
+    )
 
     re_raise_section = re.compile(
-        _re_section_template.format(r"Raises"),
-        re.X | re.S | re.M
+        _re_section_template.format(r"Raises"), re.X | re.S | re.M
     )
 
-    re_raise_line = re.compile(r"""
+    re_raise_line = re.compile(
+        r"""
         \s*  ({type}) \s* :              # identifier
         \s*  (.*)                        # beginning of optional description
-    """.format(type=re_type), re.X | re.S | re.M)
-
-    re_returns_section = re.compile(
-        _re_section_template.format(r"Returns?"),
-        re.X | re.S | re.M
+    """.format(
+            type=re_multiple_type
+        ),
+        re.X | re.S | re.M,
     )
 
-    re_returns_line = re.compile(r"""
+    re_returns_section = re.compile(
+        _re_section_template.format(r"Returns?"), re.X | re.S | re.M
+    )
+
+    re_returns_line = re.compile(
+        r"""
         \s* ({type}:)?                    # identifier
         \s* (.*)                          # beginning of description
     """.format(
-        type=re_multiple_type,
-    ), re.X | re.S | re.M)
+            type=re_multiple_type
+        ),
+        re.X | re.S | re.M,
+    )
 
-    re_property_returns_line = re.compile(r"""
+    re_property_returns_line = re.compile(
+        r"""
         ^{type}:                       # indentifier
         \s* (.*)                       # Summary line / description
     """.format(
-        type=re_multiple_type,
-    ), re.X | re.S | re.M)
+            type=re_multiple_type
+        ),
+        re.X | re.S | re.M,
+    )
 
     re_yields_section = re.compile(
-        _re_section_template.format(r"Yields?"),
-        re.X | re.S | re.M
+        _re_section_template.format(r"Yields?"), re.X | re.S | re.M
     )
 
     re_yields_line = re_returns_line
@@ -479,11 +543,13 @@ class GoogleDocstring(Docstring):
     supports_yields = True
 
     def is_valid(self):
-        return bool(self.re_param_section.search(self.doc) or
-                    self.re_raise_section.search(self.doc) or
-                    self.re_returns_section.search(self.doc) or
-                    self.re_yields_section.search(self.doc) or
-                    self.re_property_returns_line.search(self._first_line()))
+        return bool(
+            self.re_param_section.search(self.doc)
+            or self.re_raise_section.search(self.doc)
+            or self.re_returns_section.search(self.doc)
+            or self.re_yields_section.search(self.doc)
+            or self.re_property_returns_line.search(self._first_line())
+        )
 
     def has_params(self):
         if not self.doc:
@@ -527,10 +593,12 @@ class GoogleDocstring(Docstring):
         # The summary line is the return doc,
         # so the first line must not be a known directive.
         first_line = self._first_line()
-        return not bool(self.re_param_section.search(first_line) or
-                        self.re_raise_section.search(first_line) or
-                        self.re_returns_section.search(first_line) or
-                        self.re_yields_section.search(first_line))
+        return not bool(
+            self.re_param_section.search(first_line)
+            or self.re_raise_section.search(first_line)
+            or self.re_returns_section.search(first_line)
+            or self.re_yields_section.search(first_line)
+        )
 
     def has_property_type(self):
         if not self.doc:
@@ -582,7 +650,7 @@ class GoogleDocstring(Docstring):
             exc_type = match.group(1)
             exc_desc = match.group(2)
             if exc_desc:
-                types.add(exc_type)
+                types.update(_split_multiple_exc_types(exc_type))
 
         return types
 
@@ -609,7 +677,7 @@ class GoogleDocstring(Docstring):
         return params_with_doc, params_with_type
 
     def _first_line(self):
-        return self.doc.lstrip().split('\n', 1)[0]
+        return self.doc.lstrip().split("\n", 1)[0]
 
     @staticmethod
     def min_section_indent(section_match):
@@ -670,45 +738,53 @@ class NumpyDocstring(GoogleDocstring):
 
     re_param_section = re.compile(
         _re_section_template.format(r"(?:Args|Arguments|Parameters)"),
-        re.X | re.S | re.M
+        re.X | re.S | re.M,
     )
 
-    re_param_line = re.compile(r"""
+    re_param_line = re.compile(
+        r"""
         \s*  (\w+)                      # identifier
         \s*  :
         \s*  (?:({type})(?:,\s+optional)?)? # optional type declaration
         \n                              # description starts on a new line
         \s* (.*)                        # description
     """.format(
-        type=GoogleDocstring.re_multiple_type,
-    ), re.X | re.S)
+            type=GoogleDocstring.re_multiple_type
+        ),
+        re.X | re.S,
+    )
 
     re_raise_section = re.compile(
-        _re_section_template.format(r"Raises"),
-        re.X | re.S | re.M
+        _re_section_template.format(r"Raises"), re.X | re.S | re.M
     )
 
-    re_raise_line = re.compile(r"""
+    re_raise_line = re.compile(
+        r"""
         \s* ({type})$   # type declaration
         \s* (.*)        # optional description
-    """.format(type=GoogleDocstring.re_type), re.X | re.S | re.M)
-
-    re_returns_section = re.compile(
-        _re_section_template.format(r"Returns?"),
-        re.X | re.S | re.M
+    """.format(
+            type=GoogleDocstring.re_type
+        ),
+        re.X | re.S | re.M,
     )
 
-    re_returns_line = re.compile(r"""
+    re_returns_section = re.compile(
+        _re_section_template.format(r"Returns?"), re.X | re.S | re.M
+    )
+
+    re_returns_line = re.compile(
+        r"""
         \s* (?:\w+\s+:\s+)? # optional name
         ({type})$                         # type declaration
         \s* (.*)                          # optional description
     """.format(
-        type=GoogleDocstring.re_multiple_type,
-    ), re.X | re.S | re.M)
+            type=GoogleDocstring.re_multiple_type
+        ),
+        re.X | re.S | re.M,
+    )
 
     re_yields_section = re.compile(
-        _re_section_template.format(r"Yields?"),
-        re.X | re.S | re.M
+        _re_section_template.format(r"Yields?"), re.X | re.S | re.M
     )
 
     re_yields_line = re_returns_line
@@ -721,15 +797,15 @@ class NumpyDocstring(GoogleDocstring):
 
     @staticmethod
     def _is_section_header(line):
-        return bool(re.match(r'\s*-+$', line))
+        return bool(re.match(r"\s*-+$", line))
 
 
 DOCSTRING_TYPES = {
-    'sphinx': SphinxDocstring,
-    'epytext': EpytextDocstring,
-    'google': GoogleDocstring,
-    'numpy': NumpyDocstring,
-    'default': Docstring,
+    "sphinx": SphinxDocstring,
+    "epytext": EpytextDocstring,
+    "google": GoogleDocstring,
+    "numpy": NumpyDocstring,
+    "default": Docstring,
 }
 """A map of the name of the docstring type to its class.
 
